@@ -63,6 +63,7 @@ struct _WnckPagerPrivate
 
   guint dnd_activate;
   gint  dnd_workspace_number;
+  guint dnd_time;
 };
 
 enum
@@ -127,7 +128,6 @@ static AtkObject* wnck_pager_get_accessible (GtkWidget *widget);
 
 
 static gpointer parent_class;
-static guint signals[LAST_SIGNAL] = { 0 };
 
 GType
 wnck_pager_get_type (void)
@@ -149,6 +149,7 @@ wnck_pager_get_type (void)
         sizeof (WnckPager),
         0,              /* n_preallocs */
         (GInstanceInitFunc) wnck_pager_init,
+        NULL            /* value_table */
       };
       
       object_type = g_type_register_static (GTK_TYPE_WIDGET,
@@ -171,6 +172,7 @@ wnck_pager_init (WnckPager *pager)
   pager->priv->orientation = GTK_ORIENTATION_HORIZONTAL;
   pager->priv->workspace_size = 48;
   pager->priv->bg_cache = NULL;
+  pager->priv->action_window = NULL;
   pager->priv->layout_manager_token = WNCK_NO_MANAGER_TOKEN;
 
   gtk_drag_dest_set (GTK_WIDGET (pager), 0, NULL, 0, 0);
@@ -289,17 +291,27 @@ wnck_pager_size_request  (GtkWidget      *widget,
   int size;
   int n_rows;
   int focus_width;
-  
+  WnckWorkspace *space;
+
   pager = WNCK_PAGER (widget);
   
   n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
 
   g_assert (pager->priv->n_rows > 0);
   spaces_per_row = (n_spaces + pager->priv->n_rows - 1) / pager->priv->n_rows;
- 
+  space = wnck_screen_get_workspace (pager->priv->screen, 0);
+  
   if (pager->priv->orientation == GTK_ORIENTATION_VERTICAL)
     {
-      screen_aspect = (double) gdk_screen_height () / (double) gdk_screen_width ();
+      if (space) {
+        screen_aspect =
+              (double) wnck_workspace_get_height (space) /
+              (double) wnck_workspace_get_width (space);
+      } else {
+        screen_aspect =
+              (double) wnck_screen_get_height (pager->priv->screen) /
+              (double) wnck_screen_get_width (pager->priv->screen);
+      }
 
       /* TODO: Handle WNCK_PAGER_DISPLAY_NAME for this case */
 
@@ -322,10 +334,16 @@ wnck_pager_size_request  (GtkWidget      *widget,
     }
   else
     {
-      screen_aspect =
-        (double) wnck_screen_get_width (pager->priv->screen) /
-        (double) wnck_screen_get_height (pager->priv->screen);
-      
+      if (space) {
+        screen_aspect =
+              (double) wnck_workspace_get_width (space) /
+              (double) wnck_workspace_get_height (space);
+      } else {
+        screen_aspect =
+              (double) wnck_screen_get_width (pager->priv->screen) /
+              (double) wnck_screen_get_height (pager->priv->screen);
+      }
+
       if (pager->priv->show_all_workspaces)
 	{
 	  size = pager->priv->workspace_size;
@@ -876,8 +894,9 @@ wnck_pager_draw_workspace (WnckPager    *pager,
 	  WnckWindow *win = tmp->data;
 	  GdkRectangle winrect;
 	  
-	  if (pager->priv->dragging &&
-	      win == pager->priv->drag_window)
+          if ((pager->priv->dragging &&
+               win == pager->priv->drag_window) ||
+              win == pager->priv->action_window)
 	    {
 	      tmp = tmp->next;
 	      continue;
@@ -1041,6 +1060,9 @@ wnck_pager_expose_event  (GtkWidget      *widget,
                    &winrect,
                    is_current);
     }
+
+  if (pager->priv->action_window)
+    pager->priv->action_window = NULL;
   
   return FALSE;
 }
@@ -1127,7 +1149,7 @@ wnck_pager_drag_motion_timeout (gpointer data)
 
   if (dnd_workspace &&
       (pager->priv->dnd_workspace_number != wnck_workspace_get_number (active_workspace)))
-    wnck_workspace_activate (dnd_workspace);
+    wnck_workspace_activate (dnd_workspace, pager->priv->dnd_time);
 
   return FALSE;
 }
@@ -1149,6 +1171,7 @@ wnck_pager_drag_motion (GtkWidget          *widget,
                                                pager);
 
   pager->priv->dnd_workspace_number = workspace_at_point (pager, x, y, NULL, NULL);
+  pager->priv->dnd_time = time;
   gdk_drag_status (context, 0, time);
 
   return (pager->priv->dnd_workspace_number != -1);
@@ -1167,6 +1190,7 @@ wnck_pager_drag_motion_leave (GtkWidget          *widget,
     {
       g_source_remove (pager->priv->dnd_activate);
       pager->priv->dnd_workspace_number = -1;
+      pager->priv->dnd_time = 0;
       pager->priv->dnd_activate = 0;
     }
 }
@@ -1223,10 +1247,11 @@ wnck_pager_button_release (GtkWidget        *widget,
 
 	  if (space)
             {
+              pager->priv->action_window = pager->priv->drag_window;
               wnck_window_move_to_workspace (pager->priv->drag_window,
                                              space);
               if (space == wnck_screen_get_active_workspace (pager->priv->screen))
-                wnck_window_activate (pager->priv->drag_window);
+                wnck_window_activate (pager->priv->drag_window, event->time);
             }	  
 	}
       
@@ -1243,7 +1268,7 @@ wnck_pager_button_release (GtkWidget        *widget,
 
 	    /* Don't switch the desktop if we're already there */
 	    if (space != wnck_screen_get_active_workspace (pager->priv->screen))
-	      wnck_workspace_activate (space);
+	      wnck_workspace_activate (space, event->time);
 
 	    /* EWMH only lets us move the viewport for the active workspace,
 	     * but we just go ahead and hackily assume that the activate
@@ -1787,7 +1812,7 @@ wnck_pager_get_background (WnckPager *pager,
 static AtkObject *
 wnck_pager_get_accessible (GtkWidget *widget)
 {
-  gboolean static first_time = TRUE;
+  static gboolean first_time = TRUE;
 
   if (first_time) 
     {
@@ -1860,9 +1885,10 @@ _wnck_pager_get_workspace (WnckPager *pager,
 } 
 
 void 
-_wnck_pager_activate_workspace (WnckWorkspace *wspace)
+_wnck_pager_activate_workspace (WnckWorkspace *wspace,
+                                guint32        timestamp)
 {
-  wnck_workspace_activate (wspace);
+  wnck_workspace_activate (wspace, timestamp);
 }
 
 void
